@@ -7,6 +7,8 @@ from helpers import _TestSession
 import os
 import shutil
 import tempfile
+from hdfs.util import HdfsError
+from nose.tools import raises
 try:
   from hdfs.ext.dataframe import *
 except ImportError:
@@ -31,17 +33,22 @@ class TestDataframe(_TestSession):
         {'A' : 23, 'B' :  1}])
 
   def run_write_read(self, df, format, use_gzip = False, sep = '\t', 
-      index_cols = None, local_dir = None, n_threads = None):
+      index_cols = None, local_dir = None, n_threads = None,
+      hdfs_filename = 'dfreader_test', read_hdfs_filename=None, n_parts = 2):
 
     # Location on HDFS
     ext = format + ('.gz' if use_gzip else '')
-    f = '/tmp/akolchin/dfreader_test/test.' + ext # TODO: fix this
 
-    write_df(df, self.client, f, format, sep=sep, use_gzip=use_gzip, 
-      overwrite=True, num_parts=2)
+    write_df(df, self.client, hdfs_filename, format, sep=sep, use_gzip=use_gzip, 
+      overwrite=True, n_parts=n_parts)
 
-    returned_df = read_df(self.client, f, format, sep=sep, use_gzip=use_gzip, 
-      index_cols=index_cols, local_dir=local_dir, num_threads=n_threads)
+    if read_hdfs_filename is not None:
+      r_filename = read_hdfs_filename
+    else:
+      r_filename = hdfs_filename
+    returned_df = read_df(self.client, r_filename, format, sep=sep, 
+      use_gzip=use_gzip, index_cols=index_cols, local_dir=local_dir, 
+      n_threads=n_threads)
 
     assert_frame_equal(df, returned_df)
     return returned_df
@@ -49,6 +56,19 @@ class TestDataframe(_TestSession):
   def test_csv(self):
     self.run_write_read(self.test_df, format='csv')
 
+  def test_sep_suffix(self):
+    self.run_write_read(self.test_df, format='csv', 
+      hdfs_filename='dfreader_test/test/')
+
+  def test_dir_with_single_part(self):
+    self.run_write_read(self.test_df, format='csv', n_parts=1)
+
+  @raises(HdfsError)
+  def test_single_file(self):
+    self.run_write_read(self.test_df, format='csv', 
+      hdfs_filename='dfreader_test/test',
+      read_hdfs_filename='dfreader_test/test/part-r-00000')
+
   def test_csv_gz(self):
     self.run_write_read(self.test_df, format='csv', use_gzip=True)
 
